/*
Midcentury speaker and power supply case - V1.1
Featuring hidden finger joints and buried dog bones
by Robert Smith  1 Feb 2024
*/

// *** Functions and modules ***
// Calculate the number of mm given inches
function inches_to_mm(x) = 25.4 * x;

// Exclusive OR
function xor(x, y) = ((x || y) && !(x && y));

// Determine if an integer is even
function even(x) = ((abs(x) % 2) == 0);

// Calculate the largest odd integer less than the given number
function largest_odd_integer_less_than(x) = (floor(x) % 2 == 0) ? (floor(x) - 1) : floor(x);

// Calculate the radius at the vertex (which is what OpenSCAD uses) of the
// n sided approximation of a cylinder given the desired radius at the centre
// of a side and the n.
function adjusted_radius(radius, faces) = radius / cos(360 / (faces * 2));

// Create the section of the panel optionally with flange and fingers
module base_profile(width, length, flanged, fingered, step_height, router_bit_diameter) {
    half_width = width / 2;
    half_length = length / 2;
    router_bit_radius = router_bit_diameter / 2;
    adjusted_router_bit_radius = adjusted_radius(router_bit_radius, 360);
    half_dogbone_offset = half_width - step_height * 3 + router_bit_radius;

    union() {
        rotate([90, 0, 90])    
            linear_extrude(length, center = true) {
                if (flanged)
                    polygon([
                            [-half_width, 0],
                            [-half_width, step_height],
                            [-half_width + step_height * 3, step_height],
                            [-half_width + step_height * 3, step_height * 3],
                            [half_width - step_height * 3, step_height * 3],
                            [half_width - step_height * 3, step_height],
                            [half_width, step_height],
                            [half_width, 0]    
                    ]);
                else
                    polygon([
                            [-half_width + step_height, 0],
                            [-half_width + step_height, step_height],
                            [-half_width + step_height * 3, step_height],
                            [-half_width + step_height * 3, step_height * 3],
                            [half_width - step_height * 3, step_height * 3],
                            [half_width - step_height * 3, step_height],
                            [half_width - step_height, step_height],
                            [half_width - step_height, 0]   
                    ]);
            };
            
        if (fingered)
            translate([0, 0, step_height * 2])
                difference() {
                    // Finger, not quite full length
                    cube(
                        size = [length - tight_tolerance, width - step_height * 2, step_height * 2],
                        center = true
                    );
                    
                    // Dogbones
                    for (x = [-half_length, half_length])
                        for (y = [-half_dogbone_offset, half_dogbone_offset])
                            translate([x, y, 0])
                                cylinder(
                                    h = step_height * 2.2,
                                    r = adjusted_router_bit_radius,
                                    center = true,
                                    $fn = 360
                                );     
                } 
            
    }
}

// Create a panel for the box
module panel(width, length, flanged_x, flanged_y, material_thickness, router_bit_diameter) {
    // Calculate step dimensions.  We want target step width to be
    // at least four router bit diameters so that when we remove
    // the router bit radius to form the "dog bones" at either end
    // of the fingers there will be at least three router bit diameters
    // left.  The step width should also be at least the material
    // thickness so that the joints on adjacent sides don't overlap.
    step_height = material_thickness / 3;
    target_step_width = max(router_bit_diameter * 4, material_thickness * 2);

    steps_along_width = largest_odd_integer_less_than(width / target_step_width);
    steps_along_length = largest_odd_integer_less_than(length / target_step_width);
    
    step_size_along_width = width / steps_along_width;
    step_size_along_length = length / steps_along_length;
    
    // Calculate the absolute value of the min and max loop index
    step_index_width = floor(steps_along_width / 2);
    step_index_length = floor(steps_along_length / 2);    
    
    // Intersect together alternating interior and exterior and
    // exterior profiles, starting and ending with an exterior
    // profiles
    intersection() {
        for (i = [-step_index_width : step_index_width])
            translate([i * step_size_along_width, 0, 0])
                base_profile(
                    length,
                    step_size_along_width,
                    flanged_x,
                    xor(flanged_x, even(i)),
                    step_height,
                    router_bit_diameter
                );
        
        rotate([0, 0, 90])        
            for (i = [-step_index_length : step_index_length])
                translate([i * step_size_along_length, 0, 0])
                    base_profile(
                        width,
                        step_size_along_length,
                        flanged_y,
                        xor(flanged_y, even(i)),
                        step_height,
                        router_bit_diameter
                    );
    }     
}

// Create cutout for binding post
module cutout_for_binding_post(material_thickness, router_bit_diameter) {
    // Design constants in mm
    thread_length = 12;
    thread_diameter = 7.6;
    nut_inset_diameter = 20;        // Allow a bit of space to get pliers on the nut
    nut_thickness = 3.5;            // Including lock washer

    // Calculate adjusted hole radius
    adjusted_thread_radius = adjusted_radius((thread_diameter / 2) + tight_tolerance, 360);
    
    // Create a cylinder to generate the through hole
    cylinder(
        h = material_thickness * 2.2,
        r = adjusted_thread_radius,
        center = true,
        $fn = 360
    );
    
    // Calculate the amount the nut will have to be inset (if any).  This is the
    // actual material thickness minus the panel thickness the fuse holder will
    // tolerate (length of the threaded section minus the thickness of the nut).
    nut_inset_depth = material_thickness - (thread_length - nut_thickness);
    
    // If an inset is required
    if (nut_inset_depth > 0) {
        // Calculate adjusted inset radius
        adjusted_nut_inset_radius = adjusted_radius(nut_inset_diameter / 2, 360);
        
        // Create a cylinder to generate the inset
        translate([0, 0, (material_thickness - nut_inset_depth)])
            cylinder(
                h = nut_inset_depth * 1.1,
                r = adjusted_nut_inset_radius,
                center = false,
                $fn = 360
            );
    }; 
}

// Create cutout for RCA jack
module cutout_for_rca_jack(material_thickness, router_bit_diameter) {
    // Design constants in mm
    thread_length = 5.9;
    thread_diameter = 6.2;
    nut_inset_diameter = 16;        // Allow a bit of space to get pliers on the nut
    nut_thickness = 3.3;

    // Calculate adjusted hole radius
    adjusted_thread_radius = adjusted_radius((thread_diameter / 2) + tight_tolerance, 360);
    
    // Create a cylinder to generate the through hole
    cylinder(
        h = material_thickness * 2.2,
        r = adjusted_thread_radius,
        center = true,
        $fn = 360
    );
    
    // Calculate the amount the nut will have to be inset (if any).  This is the
    // actual material thickness minus the panel thickness the fuse holder will
    // tolerate (length of the threaded section minus the thickness of the nut).
    nut_inset_depth = material_thickness - (thread_length - nut_thickness);
    
    // If an inset is required
    if (nut_inset_depth > 0) {
        // Calculate adjusted inset radius
        adjusted_nut_inset_radius = adjusted_radius(nut_inset_diameter / 2, 360);
        
        // Create a cylinder to generate the inset
        translate([0, 0, (material_thickness - nut_inset_depth)])
            cylinder(
                h = nut_inset_depth * 1.1,
                r = adjusted_nut_inset_radius,
                center = false,
                $fn = 360
            );
    }; 
}

// Create cutout for power cord socket and fuse holder
module cutout_for_power_cord_socket(material_thickness, router_bit_diameter) {
    // Design constants in mm
    body_min_width = 43.1;
    body_max_width = 48.6;
    body_min_height = 18.1;
    body_max_height = 26.9;
    mounting_hole_spacing = 40;
    mounting_hole_diameter = 3;

    // Calculate useful values
    half_max_width = (body_max_width / 2) + tight_tolerance;
    half_min_height = (body_min_height / 2) + tight_tolerance;
    half_max_height = (body_max_height / 2) + tight_tolerance;
    half_spacing = mounting_hole_spacing / 2;
    
    half_min_width = half_max_width - (body_max_width - body_min_width);
    mounting_hole_radius = max(mounting_hole_diameter, router_bit_diameter) / 2;
    adjusted_mounting_hole_radius = adjusted_radius(mounting_hole_radius + tight_tolerance, 360);
    
    // Create a polygon which will generate a cutout for the body of the socket
    linear_extrude(material_thickness * 2.2, center = true) 
        polygon([
            [-half_max_width, half_min_height],
            [-half_min_width, half_max_height],
            [half_max_width, half_max_height],
            [half_max_width, -half_max_height],
            [-half_min_width, -half_max_height],
            [-half_max_width, -half_min_height]
            ]);

    // For each of two mounting holes
    for (y = [-half_spacing, half_spacing]) {
        // Create a cylinder which will generate the hole
        translate([0, y, 0])
            cylinder(
                h = material_thickness * 2.2,
                r = adjusted_mounting_hole_radius,
                center = true,
                $fn = 360
            );
    }
}

// Create cutout for Meanwell 12V 12.5A power supply
module cutout_for_power_supply(material_thickness, router_bit_diameter) {
    // Design constants in mm
    mounting_hole_pattern_width = 153;
    mounting_hole_pattern_depth = 86;
    mounting_hole_diameter = 2.78;
    
    // Calculate mounting hole diameter
    half_width = mounting_hole_pattern_width / 2;
    half_depth = mounting_hole_pattern_depth / 2;
    mounting_hole_radius = max(mounting_hole_diameter, router_bit_diameter) / 2;
    adjusted_mounting_hole_radius = adjusted_radius(mounting_hole_radius + tight_tolerance, 360);
      
    // Create a cylinder which will generate the hole
    translate([-half_width, half_depth, 0])
        cylinder(
            h = material_thickness * 2.2,
            r = adjusted_mounting_hole_radius,
            center = true,
            $fn = 360
        );
    
    // Create a cylinder which will generate the hole
    translate([half_width, -half_depth, 0])
        cylinder(
            h = material_thickness * 2.2,
            r = adjusted_mounting_hole_radius,
            center = true,
            $fn = 360
            );    
}

// Create cutout for speaker Digikey 2856-FR7-4OHM-ND
module cutout_for_speaker(material_thickness, router_bit_diameter) {
    // Design constants in mm
    cone_diameter = 66;
    mounting_circle_diameter = 74;
    mounting_hole_diameter = 4.2;
    
    // Calculate mounting hole diameter
    mounting_circle_radius = mounting_circle_diameter / 2;
    mounting_hole_radius = max(mounting_hole_diameter, router_bit_diameter) / 2;
    adjusted_mounting_hole_radius = adjusted_radius(mounting_hole_radius + tight_tolerance, 360);
    
    // For each of four mounting holes
    for (angle = [45 : 90 : 315]) {
        // Calculate position of mounting hole
        x = cos(angle) * mounting_circle_radius;
        y = sin(angle) * mounting_circle_radius;
        
        // Create a cylinder which will generate the hole
        translate([x, y, 0])
            cylinder(
                h = material_thickness * 2.2,
                r = adjusted_mounting_hole_radius,
                center = true,
                $fn = 360
            );
    }
   
    // Calculate the maximum grill slot index by dividing the speaker
    // cone radius by twice the router bit diameter based on the assumption
    // that the slot and adjacent remaining material will both be one router
    // bit diameter in width.
    max_index = floor((cone_diameter / 2) / (router_bit_diameter * 2));
    
    // Pre-calculate a couple of things so that we don't have to recalculate
    // them on every pass through the loop
    cone_radius_squared = pow((cone_diameter / 2), 2);
    router_bit_radius = router_bit_diameter / 2;
    
    // Increase the slot size ever so slightly to avoid problems slicing
    adjusted_router_bit_diameter = router_bit_diameter + tight_tolerance;
    
    // For each grill slot
    for (index = [-max_index: max_index]) {
        // Calculate the y axis position of the grill slot
        y = index * router_bit_diameter * 2;
       
        // Calculate the width of one half of the grill slot assuming a solid
        // portion one router bit diameter wide separating the two halves
        width = sqrt(cone_radius_squared - pow(y, 2)) - router_bit_radius;
        
        // Calculate the offset of the half slot
        x = (width / 2) + router_bit_radius;
       
        // Create two cubes which will generate the two halves of the slot
        translate([-x, y, 0])
            cube([width, adjusted_router_bit_diameter, material_thickness * 2.2], center = true);
        
        translate([x, y, 0])
            cube([width, adjusted_router_bit_diameter, material_thickness * 2.2], center = true);       
    }
}

// Create cutout for ventillation louvres
module cutout_for_ventillation(material_thickness, router_bit_diameter, width, depth) {
    
    // Calculate the maximum grill slot index by dividing the desired depth
    // radius by twice the router bit diameter based on the assumption that
    // the slot and adjacent remaining material will both be one router
    // bit diameter in width.
    max_index = floor((depth / 2) / (router_bit_diameter * 2));
    
    // Pre-calculate a couple of things so that we don't have to recalculate
    // them on every pass through the loop
    router_bit_radius = router_bit_diameter / 2;
    
    // Increase the slot size ever so slightly to avoid problems slicing
    adjusted_router_bit_diameter = router_bit_diameter + tight_tolerance;
    
    // For each ventillation louvre slot
    for (index = [-max_index: max_index]) {
        // Calculate the y axis position of the grill slot
        y = index * router_bit_diameter * 2;
             
        // Create two cubes which will generate the two halves of the slot
        translate([0, y, 0])
            cube([width, adjusted_router_bit_diameter, material_thickness * 2.2], center = true);
        
        translate([0, y, 0])
            cube([width, adjusted_router_bit_diameter, material_thickness * 2.2], center = true);       
    }
}

// Create application specific cutouts for a given panel
module cutouts(index, material_thickness, router_bit_diameter, box_width, box_depth, box_height) {
    
    if (index == 0) {                             // left
        // Calculate the x offset of ventillation louvre
        x = (box_width - (material_thickness * 2)) / 16;

       // Cutout for ventillation louvre
       translate([x * 3, 0, 0])
           cutout_for_ventillation(material_thickness, router_bit_diameter, 20, 40);        
    }
    else
       if (index == 1) {                          // bottom          
       }
       else
            if (index == 2) {                     // right            
                 // Calculate the x and y offset of ventillation louvre and power socket
                 x = (box_width - (material_thickness * 2)) / 16;

                // Cutout for ventillation louvre
                translate([x * -3, 0, 0])
                    cutout_for_ventillation(material_thickness, router_bit_diameter, 20, 40);
            
                // Cutout for power socket and fuse
                translate([x * 2, 0, 0])
                    rotate([0, 0, 180])
                        cutout_for_power_cord_socket(material_thickness, router_bit_diameter);
           }
            else
                if (index == 3) {                 // top
                }
                else
                    if (index == 4) {             // back
                        // Cutout for the power supply
                        cutout_for_power_supply(material_thickness, router_bit_diameter);                }
                    else
                        if (index == 5) {         // front
                            // Calculate the x and y offset of speakers
                            x = (box_width - (material_thickness * 2)) / 32;
                            y = (box_height - (material_thickness * 2)) / 32;
                            
                            // Cutout for speaker
                            translate([x * 5, y * -4, 0])
                                cutout_for_speaker(material_thickness, router_bit_diameter);

                            // Cutout for speaker
                            translate([x * -9, y * -4, 0])
                                cutout_for_speaker(material_thickness, router_bit_diameter);
                  
                            // Cutout for an RCA jack
                            translate([x * -7, y * 11, 0])
                                cutout_for_rca_jack(material_thickness, router_bit_diameter);
                            
                            // Cutout for an RCA jack
                            translate([x * -11, y * 11, 0])
                                cutout_for_rca_jack(material_thickness, router_bit_diameter);
                        
                            // Cutout for a binding post
                            translate([x * 3, y * 11, 0])
                                cutout_for_binding_post(material_thickness, router_bit_diameter);
                            
                            // Cutout for a binding post
                            translate([x * 7, y * 11, 0])
                                cutout_for_binding_post(material_thickness, router_bit_diameter);                    }

};

// Create a box by looping through the panels and applying a view-specific translation
// and rotation.
module box(material_thickness, router_bit_diameter, box_width, box_depth, box_height, view) {     
    // Define width, length and goofiness of panels
    panel_parm = [
        [box_height, box_depth, false, false],          // left
        [box_width, box_depth, true, true],             // bottom
        [box_height, box_depth, false, false],          // right
        [box_width, box_depth, true, true],             // top
        [box_width, box_height, false, true],           // back
        [box_width, box_height, false, true]            // front
    ];
 
    // Define spacing between parts so that when the router has passed around the
    // outside of adjacent parts, the remaining material will be as wide as the
    // material is thick.  This is just a rule of thumb and can be adjusted.
    spacing = material_thickness + (router_bit_diameter * 2);
    
    // Calculate half sizes
    half_width = box_width / 2;
    half_depth = box_depth / 2;
    half_height = box_height / 2;
    
    // Define translation of panels by view
    panel_translation = [
        // flattened
        [
            [-(half_width + half_height + spacing), 0, 0],                                    // left
            [0, 0, 0],                                                                        // bottom
            [(half_width + half_height + spacing), 0, 0],                                     // right
            [(box_width + box_height + spacing * 2), 0, 0],                                   // top
            [0, (half_depth + half_height + spacing), 0],                                     // back
            [0, -(half_depth + half_height + spacing), 0]                                     // front
        ],
        
        // machining
        [
            [(half_width + half_depth + spacing), (half_depth + half_height + spacing), 0],   // left
            [0, 0, 0],                                                                        // bottom
            [(half_width + half_height + spacing), 0, 0],                                     // right
            [(box_width + box_height + spacing * 2), 0, 0],                                   // top
            [0, (half_depth + half_height + spacing), 0],                                     // back
            [(box_width + box_depth + spacing * 2), (half_depth + half_height + spacing), 0]  // front
        ],
         
        // assembled        
        [
            [-(half_width + spacing), 0, 0],                                                  // left
            [0, 0, -(half_height + spacing)],                                                 // bottom
            [(half_width + spacing), 0, 0],                                                   // right
            [0, 0, (half_height + spacing)],                                                  // top
            [0, (half_depth + spacing), 0],                                                   // back
            [0, -(half_depth + spacing), 0]                                                   // front
        ]        
    ];

    // Define rotation of panels by view
    panel_rotation = [
        // flattened
        [                   // flattened
            [0, 0, 0],      // left
            [0, 0, 0],      // bottom
            [0, 0, 180],    // right
            [0, 0, 180],    // top
            [0, 0, 0],      // back
            [0, 0, 0]       // front            
        ],
        
        // machining
        [
            [0, 0, 90],     // left
            [0, 0, 0],      // bottom
            [0, 0, 180],    // right
            [0, 0, 180],    // top
            [0, 0, 0],      // back
            [0, 0, 0]       // front            
        ],
                           
        // assembled
        [
            [0, 90, 0],     // left
            [0, 0, 0],      // bottom
            [0, 90, 180],   // right
            [0, 180, 180],  // top
            [90, 0, 0],     // back
            [270, 0, 0]     // front           
        ],       
    ];        
    
    // For each panel
    for (index = [0 : 5])
        translate(panel_translation[view][index])
            rotate(panel_rotation[view][index])
                difference() {
                    // Base panel
                    panel(
                        panel_parm[index][0],
                        panel_parm[index][1],
                        panel_parm[index][2],
                        panel_parm[index][3],
                        material_thickness,
                        router_bit_diameter
                    );
                    
                    // Application specific cutouts
                    cutouts(
                        index,
                        material_thickness,
                        router_bit_diameter,
                        box_width,
                        box_depth,
                        box_height);
                }
}

/*** Constants ***/
// Tolerance in mm for mating parts
tight_tolerance = inches_to_mm(0.01);

// Define view constants
flattened = 0;
machining = 1;
assembled = 2;

/*** Design parameters in mm ***/
material_thickness = 5.83;
router_bit_diameter = inches_to_mm(0.127);

// Box outside dimensions
box_width = inches_to_mm(8);
box_depth = inches_to_mm(4);
box_height = inches_to_mm(5.25);


// Generate box with specified view
box(material_thickness, router_bit_diameter, box_width, box_depth, box_height, machining);

